# Copyright (C) 2012 Team tiramisu (see AUTHORS for all contributors)
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
# The original `Config` design model is unproudly borrowed from
# the rough gus of pypy: pypy: http://codespeak.net/svn/pypy/dist/pypy/config/
# the whole pypy projet is under MIT licence
# ____________________________________________________________
"enables us to carry out a calculation and return an option's value"
from tiramisu.error import PropertiesOptionError, ConflictConfigError
# ____________________________________________________________
# automatic Option object
#def special_owner_factory(name, owner, value,
#                          callback, callback_params=None, config=None):
#    # in case of an 'auto' and a 'fill' without a value,
#    # we have to carry out a calculation
#    return calc_factory(name, callback, callback_params, config)
def carry_out_calculation(name, option, config):
    # FIXME we have to know the exact status of the config
    # not to disrupt it
    # config.freeze()
    callback=option.getcallback()
    callback_params=option.getcallback_params()
    if callback_params is None:
        callback_params = {}
    tcparams = {}
    one_is_multi = False
    len_multi = 0

    for key, values in callback_params.items():
        for value in values:
            if type(value) == tuple:
                path, check_disabled = value
                try:
                    opt_value = config._getattr(path, permissive=True)
                    opt = config.unwrap_from_path(path)
                except PropertiesOptionError, err:
                    if check_disabled:
                        continue
                    raise PropertiesOptionError(err, err.proptype)
                is_multi = opt.is_multi()
                if is_multi:
                    if opt_value != None:
                        len_value = len(opt_value)
                        if len_multi != 0 and len_multi != len_value:
                            raise ConflictConfigError('unable to carry out a calculation, '
                            'option values with multi types must have same length for: '
                             + name)
                        len_multi = len_value
                    one_is_multi = True
                tcparams.setdefault(key, []).append((opt_value, is_multi))
            else:
                tcparams.setdefault(key, []).append((value, False))

    if one_is_multi:
        ret = []
        for incr in range(len_multi):
            tcp = {}
            params = []
            for key, couples in tcparams.items():
                for couple in couples:
                    value, ismulti = couple
                    if ismulti and value != None:
                        if key == '':
                            params.append(value[incr])
                        else:
                            if len(value) > incr:
                                tcp[key] = value[incr]
                            else:
                                tcp[key] = ''
                    else:
                        if key == '':
                            params.append(value)
                        else:
                            tcp[key] = value
            ret.append(calculate(name, callback, params, tcp))
        return ret
    else:
        tcp = {}
        params = []
        for key, couples in tcparams.items():
            for couple in couples:
                if key == '':
                    value = couple[0]
                    params.append(value)
                else:
                    tcp[key] = couple[0]
        return calculate(name, callback, params, tcp)

def calculate(name, callback, params, tcparams):
    try:
        # XXX not only creole...
        from creole import eosfunc
        return getattr(eosfunc, callback)(*params, **tcparams)
    except AttributeError, err:
        import traceback
        traceback.print_exc()
        raise ConflictConfigError("callback: {0} return error {1} for "
                       "option: {2}".format(callback, str(err), name))

